#include "common.h"

struct __declspec(uuid("9C6B4CB0-23F8-49CC-A3ED-45A55000A6D2")) IDxDiagProvider;
struct __declspec(uuid("7D0F462F-4064-4862-BC7F-933E5058C10F")) IDxDiagContainer;

//-----------------------------------------------------------------------------
// External function-prototypes
//-----------------------------------------------------------------------------
extern HRESULT GetDXVersion( DWORD* pdwDirectXVersion, TCHAR* strDirectXVersion, int cchDirectXVersion );

#include "resource.h"

//-----------------------------------------------------------------------------
// WinMain
//-----------------------------------------------------------------------------
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, int );

struct FrameworkVersionInfo
{
	int major;
	int minor;
	int build;
};

//-----------------------------------------------------------------------------
// DialogNX
//-----------------------------------------------------------------------------
class CRuntimeInfoDialog : public CDialogImpl<CRuntimeInfoDialog>
{
public:
	// ATL ̗v
	enum { IDD = IDD_RUNTIMEINFO_DIALOG };

	BEGIN_MSG_MAP(CScreenDialog)
		MESSAGE_HANDLER( WM_INITDIALOG,     OnInitDialog )
		MESSAGE_HANDLER( WM_DESTROY,        OnDestroy )
		MESSAGE_HANDLER( WM_CLOSE,          OnClose )
	END_MSG_MAP()

	//-------------------------------------------------------------------------
	// OnInit handler
	//-------------------------------------------------------------------------
	LRESULT OnInitDialog( UINT msg, WPARAM wParam, LPARAM lParam, BOOL& handled )
	{
		CenterWindow();

		directXEdit = GetDlgItem( IDC_EDIT1 );
		dotNetEdit = GetDlgItem( IDC_EDIT2 );
		managedDirectXEdit = GetDlgItem( IDC_EDIT3 );

		CheckFrameworkVersion();
		CheckDirectXVersion();
		CheckMDXVersion();
		
		return 0L;
	}

	void CheckMDXVersion()
	{
		typedef CComPtr<IDxDiagProvider> IDxDiagProviderPtr;
		typedef CComPtr<IDxDiagContainer> IDxDiagContainerPtr;

		// voC_
		IDxDiagProviderPtr provider;
		{
			provider.CoCreateInstance( CLSID_DxDiagProvider );
			if( !provider )
			{
				return;
			}
		}

		// 
		{
			DXDIAG_INIT_PARAMS initParam;
			ZeroMemory( &initParam, sizeof(DXDIAG_INIT_PARAMS) );
			initParam.dwSize                  = sizeof(DXDIAG_INIT_PARAMS);
			initParam.dwDxDiagHeaderVersion   = DXDIAG_DX9_SDK_VERSION;
			initParam.bAllowWHQLChecks        = false;
			initParam.pReserved               = NULL;

			if( FAILED( provider->Initialize( &initParam ) ) )
			{
				return;
			}
		}

		// [gRei
		IDxDiagContainerPtr root;
		if( FAILED( provider->GetRootContainer( &root ) ) )
		{
			return;
		}

		// DirectXFiles Rei
		IDxDiagContainerPtr directXFiles;
		if( FAILED( root->GetChildContainer( L"DxDiag_DirectXFiles", &directXFiles ) ) )
		{
			return;
		}

		DWORD numFiles = 0;
		if( FAILED( directXFiles->GetNumberOfChildContainers( &numFiles ) ) )
		{
			return;
		}

		std::vector<CString> result;

		for( DWORD i = 0; i < numFiles; ++i )
		{
			// DirectXFiles ̃TuRei̖O擾
			const int nameLen = 1024;
			wchar_t nameBuf[nameLen];
			if( FAILED( directXFiles->EnumChildContainerNames( i, nameBuf, nameLen ) ) )
			{
				return;
			}

			// TuReiTuRei擾
			IDxDiagContainerPtr file;
			if( FAILED( directXFiles->GetChildContainer( nameBuf, &file ) ) )
			{
				return;
			}

			// ꂪ "Microsoft.DirectX.dll" ł΃o[WL^
			CComVariant fileName;
			if( SUCCEEDED( file->GetProp( L"szName", &fileName ) ) )
			{
				if( CString( fileName.bstrVal ).MakeLower() == L"microsoft.directx.dll" )
				{
					CComVariant version;
					if( SUCCEEDED( file->GetProp( L"szVersion", &version ) ) )
					{
						result.push_back( CString( version.bstrVal ) );
					}
				}
			}
		}

		// _CAOɕ\
		CString mdxRuntimes;
		for(std::vector<CString>::const_iterator i = result.begin(); i != result.end(); ++i )
		{
			mdxRuntimes += "[";
			mdxRuntimes += *i;
			mdxRuntimes += "] ";
		}
		managedDirectXEdit.SetWindowText( mdxRuntimes );
	}


	void CheckDirectXVersion()
	{
		DWORD ver = 0;
		const int num = 1024;
		TCHAR verstr[num];

		// _CAOɕ\
		if( SUCCEEDED( GetDXVersion( &ver, verstr, num ) ) )
		{
			directXEdit.SetWindowText( verstr );
		}
	}
	void CheckFrameworkVersion()
	{
		std::vector<FrameworkVersionInfo> versions;

		CString frameforkDir;
		frameforkDir.GetEnvironmentVariable("WINDIR");
		frameforkDir += "\\Microsoft.Net\\Framework\\";

		WIN32_FIND_DATA findFileData;
		HANDLE handle = FindFirstFile( (frameforkDir + "*").GetBuffer(), &findFileData);

		if (handle == INVALID_HANDLE_VALUE)
		{
			return;
		}
		else
		{
			CAtlRegExp<> regExp;
			REParseError status = regExp.Parse( "^[vV]{\\d+}\\.{\\d+}\\.{\\d+}$" );
			while(true)
			{
				if( (findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) != 0 )
				{
					CString dirName( findFileData.cFileName );
					CAtlREMatchContext<> result;
					if (regExp.Match(dirName, &result) && result.m_uNumGroups == 3)
					{
						// ACXg[ς݂̃fBNgcĂꍇ̂ŁC
						// System.dll 邩`FbN
						CAtlFile systemDLL;
						if( SUCCEEDED( systemDLL.Create( frameforkDir + dirName + "\\System.dll", 0, FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_EXISTING ) ) )
						{
							FrameworkVersionInfo verinfo;
							std::vector<int> digits;
							for (DWORD index = 0; index < result.m_uNumGroups; ++index)
							{
								const CAtlREMatchContext<>::RECHAR* szStart = 0;
								const CAtlREMatchContext<>::RECHAR* szEnd = 0;
								result.GetMatch(index, &szStart, &szEnd);
								const int num = atoi( CString( szStart, (szEnd - szStart)/sizeof(CAtlREMatchContext<>::RECHAR) ) );
								digits.push_back( num ); 
							}
							verinfo.major = digits[0];
							verinfo.minor = digits[1];
							verinfo.build = digits[2];
							versions.push_back( verinfo );
						}
					}

				}
				if( FindNextFile( handle, &findFileData ) != TRUE )
				{
					const DWORD error = GetLastError();
					if( error == ERROR_NO_MORE_FILES )
					{
						FindClose(handle);
						break;
					}
					else
					{
						CString msg;
						msg.Format( "Invalid File Handle. Get Last Error reports %d\n", error );
						throw msg;
					}
				}
			}
		}

		// _CAOɕ\
		CString dotnetFrameworkRuntimes;
		for( std::vector<FrameworkVersionInfo>::const_iterator i = versions.begin();
				i != versions.end(); ++i )
		{
			const FrameworkVersionInfo& info = *i;
			CString vermsg;
			vermsg.Format( "[%d.%d.%d] ", info.major, info.minor, info.build );
			dotnetFrameworkRuntimes += vermsg;
		}
		dotNetEdit.SetWindowText( dotnetFrameworkRuntimes );
	}

	//-------------------------------------------------------------------------
	// OnDestroy handler
	//-------------------------------------------------------------------------
	LRESULT OnDestroy( UINT msg, WPARAM wParam, LPARAM lParam, BOOL& handled )
	{
		directXEdit.Detach();
		dotNetEdit.Detach();
		managedDirectXEdit.Detach();
		return 0L;
	}

	//-------------------------------------------------------------------------
	// OnClose handler
	//-------------------------------------------------------------------------
	LRESULT OnClose( UINT msg, WPARAM wParam, LPARAM lParam, BOOL& handled )
	{
		EndDialog(0);
		return 0L;
	}

	//-------------------------------------------------------------------------
	// Constructor
	//-------------------------------------------------------------------------
	CRuntimeInfoDialog()
	{
	}
	
private:
	CWindow directXEdit;
	CWindow dotNetEdit;
	CWindow managedDirectXEdit;
};


//-----------------------------------------------------------------------------
// WinMain
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int )
{
	// ATLW[
	CAtlWinModule module;
	{
		// COM 
		CoInitialize( NULL );
		{
			CRuntimeInfoDialog dialog;
			
			// e평ݒ
			dialog.DoModal();
		}
		CoUninitialize();

		return 0;
	}
}
